package tech.mercantec.easyeat.ui.shopping_list

import android.app.AlertDialog
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ArrayAdapter
import android.widget.EditText
import android.widget.PopupMenu
import android.widget.Spinner
import android.widget.TextView
import android.widget.Toast
import androidx.fragment.app.Fragment
import tech.mercantec.easyeat.R
import tech.mercantec.easyeat.databinding.FragmentShoppingListBinding
import tech.mercantec.easyeat.helpers.ApiRequestException
import tech.mercantec.easyeat.helpers.addShoppingItem
import tech.mercantec.easyeat.helpers.deleteShoppingItem
import tech.mercantec.easyeat.helpers.editShoppingItem
import tech.mercantec.easyeat.helpers.getShoppingList
import tech.mercantec.easyeat.helpers.toggleShoppingItemChecked
import tech.mercantec.easyeat.models.ShoppingListItem
import java.util.ArrayList
import kotlin.concurrent.thread

class ShoppingListFragment : Fragment() {
    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        val binding = FragmentShoppingListBinding.inflate(inflater, container, false)

        binding.shoppingList.visibility = View.GONE
        binding.emptyShoppingList.visibility = View.GONE
        binding.addToShoppingList.visibility = View.GONE
        binding.loading.visibility = View.VISIBLE

        // Fetch shopping list items
        thread {
            val items: Array<ShoppingListItem>
            try {
                items = getShoppingList(requireContext())
            } catch (e: ApiRequestException) {
                activity?.runOnUiThread {
                    Toast.makeText(context, e.message, Toast.LENGTH_LONG).show()

                    binding.loading.visibility = View.GONE
                }

                return@thread
            }

            activity?.runOnUiThread {
                binding.shoppingList.visibility = View.VISIBLE
                binding.emptyShoppingList.visibility = View.VISIBLE
                binding.addToShoppingList.visibility = View.VISIBLE
                binding.loading.visibility = View.GONE

                binding.shoppingList.emptyView = binding.emptyShoppingList
                binding.shoppingList.adapter = ShoppingItemAdapter(requireContext(), ArrayList(items.toMutableList()))
            }
        }

        // Check / uncheck when clicking shopping item
        binding.shoppingList.setOnItemClickListener { parent, view, position, id ->
            val item = parent.getItemAtPosition(position) as ShoppingListItem
            item.checked = !item.checked

            thread {
                try {
                    toggleShoppingItemChecked(requireContext(), item)
                } catch (e: ApiRequestException) {
                    Toast.makeText(requireContext(), e.message, Toast.LENGTH_LONG).show()
                }
            }

            val adapter = parent.adapter as ShoppingItemAdapter

            adapter.remove(item)
            adapter.insert(item, position)
        }

        // Show context menu when long clicking shopping item
        binding.shoppingList.setOnItemLongClickListener { parent, view, position, id ->
            val item = parent.getItemAtPosition(position) as ShoppingListItem

            val popup = PopupMenu(requireActivity(), view)
            popup.apply {
                menuInflater.inflate(R.menu.shopping_item_context_menu, menu)

                setOnMenuItemClickListener {
                    when (it.itemId) {
                        R.id.edit_shopping_item -> {
                            showEditDialog(item) { dialog, newItem ->
                                thread {
                                    try {
                                        editShoppingItem(requireContext(), item, newItem)
                                    } catch (e: ApiRequestException) {
                                        activity?.runOnUiThread {
                                            Toast.makeText(context, e.message, Toast.LENGTH_LONG).show()
                                        }

                                        return@thread
                                    } finally {
                                        dialog.dismiss()
                                    }

                                    activity?.runOnUiThread {
                                        val adapter = binding.shoppingList.adapter as ShoppingItemAdapter

                                        adapter.remove(item)
                                        adapter.insert(newItem, position)
                                    }
                                }
                            }

                            true
                        }
                        R.id.remove_shopping_item -> {
                            (parent.adapter as ShoppingItemAdapter).remove(item)

                            thread {
                                try {
                                    deleteShoppingItem(requireContext(), item)
                                } catch (e: ApiRequestException) {
                                    activity?.runOnUiThread {
                                        Toast.makeText(context, e.message, Toast.LENGTH_LONG).show()
                                    }
                                }
                            }

                            true
                        }
                        else -> false
                    }
                }

                show()
            }


            return@setOnItemLongClickListener true
        }

        // Show new item dialog when clicking add
        binding.addToShoppingList.setOnClickListener {
            showEditDialog(null) { dialog, item ->
                thread {
                    val newItem: ShoppingListItem
                    try {
                        newItem = addShoppingItem(requireContext(), item)
                    } catch (e: ApiRequestException) {
                        activity?.runOnUiThread {
                            Toast.makeText(context, e.message, Toast.LENGTH_LONG).show()
                        }

                        return@thread
                    } finally {
                        activity?.runOnUiThread {
                            dialog.dismiss()
                        }
                    }

                    activity?.runOnUiThread {
                        val adapter = binding.shoppingList.adapter as ShoppingItemAdapter

                        for (i in 0..<adapter.count) {
                            if (adapter.getItem(i)?.id == newItem.id) {
                                adapter.remove(adapter.getItem(i))
                                adapter.insert(newItem, i)

                                return@runOnUiThread
                            }
                        }

                        adapter.insert(newItem, adapter.count)
                    }
                }
            }
        }

        return binding.root
    }

    private fun showEditDialog(item: ShoppingListItem?, onSave: (dialog: AlertDialog, item: ShoppingListItem) -> Unit) {
        val view = requireActivity().layoutInflater.inflate(R.layout.dialog_edit_shopping_item, null)

        val dialog = AlertDialog.Builder(activity)
            .setView(view)
            .setPositiveButton(R.string.add_label) { dialog, id ->
                val dialog = dialog as AlertDialog

                val amount = view.findViewById<EditText>(R.id.amount).text.toString().toDouble()
                val unit = view.findViewById<Spinner>(R.id.unit_selector).selectedItem.toString()
                    .ifEmpty { null }
                val name = view.findViewById<EditText>(R.id.name).text.toString()

                onSave(dialog, ShoppingListItem(item?.id, name, amount, unit, item?.checked ?: false))

                dialog.getButton(AlertDialog.BUTTON_POSITIVE).isEnabled = false
                dialog.getButton(AlertDialog.BUTTON_NEGATIVE).isEnabled = false


            }
            .setNegativeButton(R.string.cancel_label) { dialog, _ ->
                dialog.cancel()
            }
            .create()

        dialog.show()

        val adapter = ArrayAdapter.createFromResource(
            requireContext(),
            R.array.units,
            android.R.layout.simple_spinner_item
        )
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)

        dialog.findViewById<Spinner>(R.id.unit_selector).adapter = adapter

        // Pre-fill dialog inputs with current item if applicable
        item?.let { item ->
            view.findViewById<EditText>(R.id.amount).setText(item.amount?.toBigDecimal()?.stripTrailingZeros()?.toPlainString(), TextView.BufferType.EDITABLE)
            view.findViewById<Spinner>(R.id.unit_selector).setSelection(adapter.getPosition(item.unit))
            view.findViewById<EditText>(R.id.name).setText(item.name, TextView.BufferType.EDITABLE)

            view.findViewById<TextView>(R.id.title).text = resources.getString(R.string.edit_shopping_item_label)
            dialog.getButton(AlertDialog.BUTTON_POSITIVE).text = resources.getString(R.string.save_label)
        }
    }
}